@@ -38,6 +38,7 @@
 # -link.largest.ontop  pass to `chordDiagramFromMatrix` or `chordDiagramFromDataFrame`
 # -link.visible pass to `chordDiagramFromMatrix` or `chordDiagramFromDataFrame`
 # -link.rank order to add links to the circle, a large value means to add it later.
+# -link.overlap pass to `chordDiagramFromMatrix` or `chordDiagramFromDataFrame`
 # -scale scale each sector to same width
 # -big.gap Gap between the two sets of sectors. If the input is a matrix, the two sets
 #      are row sectors and column sectors. If the input is a data frame, the two sets
@@ -84,13 +85,19 @@ chordDiagram = function(x, grid.col = NULL, grid.border = NA, transparency = 0.5
 	link.arr.type = "triangle", link.arr.lty = par("lty"), 
 	link.arr.lwd = par("lwd"), link.arr.col = par("col"), 
 	link.largest.ontop = FALSE, link.visible = TRUE, 
-	link.rank = NULL, scale = FALSE, big.gap = 10, small.gap = 1, ...) {
+	link.rank = NULL, link.overlap = FALSE, 
+	scale = FALSE, big.gap = 10, small.gap = 1, ...) {
 		
 	if(inherits(x, "table"))  {
 		if(length(dim(x)) == 2) {
 			class(x) = "matrix"
 		}
 	}
+	if(inherits(x, "matrix")) {
+		if(ncol(x) == 2 && nrow(x) > 10) {
+			x = as.data.frame(x)
+		}
+	}
 
 	if(inherits(x, "matrix")) {
 		chordDiagramFromMatrix(x, grid.col = grid.col, grid.border = grid.border, transparency = transparency,
@@ -101,9 +108,9 @@ chordDiagram = function(x, grid.col = NULL, grid.border = NA, transparency = 0.5
 			link.border = link.border, link.lwd = link.lwd, link.lty = link.lty, link.sort = link.sort, link.decreasing = link.decreasing,
 			link.arr.length = link.arr.length, link.arr.width = link.arr.width, link.arr.type = link.arr.type, link.arr.lty = link.arr.lty,
 			link.arr.lwd = link.arr.lwd, link.arr.col = link.arr.col, link.largest.ontop = link.largest.ontop, 
-			link.visible = link.visible, link.rank = link.rank, scale = scale, big.gap = big.gap, small.gap = small.gap, ...)
-	} else if(inherits(x, "data.frame")) {
-		x = as.data.frame(x)
+			link.visible = link.visible, link.rank = link.rank, link.overlap = link.overlap, scale = scale, big.gap = big.gap, small.gap = small.gap, ...)
+	} else {
+		x = validate_data_frame(x)
 		if(ncol(x) > 3) {
 			if(all(sapply(x, inherits, c("numeric", "integer")))) {
 				warning("It seems your input data is an adjacency matrix, maybe you need to convert it to 'matrix' explicitely.")
@@ -115,7 +122,7 @@ chordDiagram = function(x, grid.col = NULL, grid.border = NA, transparency = 0.5
 					link.border = link.border, link.lwd = link.lwd, link.lty = link.lty, link.sort = link.sort, link.decreasing = link.decreasing,
 					link.arr.length = link.arr.length, link.arr.width = link.arr.width, link.arr.type = link.arr.type, link.arr.lty = link.arr.lty,
 					link.arr.lwd = link.arr.lwd, link.arr.col = link.arr.col, link.largest.ontop = link.largest.ontop, 
-					link.visible = link.visible, link.rank = link.rank, scale = scale, big.gap = big.gap, small.gap = small.gap, ...)))
+					link.visible = link.visible, link.rank = link.rank, link.overlap = link.overlap, scale = scale, big.gap = big.gap, small.gap = small.gap, ...)))
 			} else {
 				chordDiagramFromDataFrame(x, grid.col = grid.col, grid.border = grid.border, transparency = transparency,
 					col = col, order = order, directional = directional, direction.type = direction.type,
@@ -124,7 +131,7 @@ chordDiagram = function(x, grid.col = NULL, grid.border = NA, transparency = 0.5
 					link.border = link.border, link.lwd = link.lwd, link.lty = link.lty, link.sort = link.sort, link.decreasing = link.decreasing,
 					link.arr.length = link.arr.length, link.arr.width = link.arr.width, link.arr.type = link.arr.type, link.arr.lty = link.arr.lty,
 					link.arr.lwd = link.arr.lwd, link.arr.col = link.arr.col, link.largest.ontop = link.largest.ontop, 
-					link.visible = link.visible, link.rank = link.rank, scale = scale, big.gap = big.gap, small.gap = small.gap, ...)
+					link.visible = link.visible, link.rank = link.rank, link.overlap = link.overlap, scale = scale, big.gap = big.gap, small.gap = small.gap, ...)
 			}
 		} else {
 			chordDiagramFromDataFrame(x, grid.col = grid.col, grid.border = grid.border, transparency = transparency,
@@ -134,12 +141,9 @@ chordDiagram = function(x, grid.col = NULL, grid.border = NA, transparency = 0.5
 				link.border = link.border, link.lwd = link.lwd, link.lty = link.lty, link.sort = link.sort, link.decreasing = link.decreasing,
 				link.arr.length = link.arr.length, link.arr.width = link.arr.width, link.arr.type = link.arr.type, link.arr.lty = link.arr.lty,
 				link.arr.lwd = link.arr.lwd, link.arr.col = link.arr.col, link.largest.ontop = link.largest.ontop, 
-				link.visible = link.visible, link.rank = link.rank, scale = scale, big.gap = big.gap, small.gap = small.gap, ...)
+				link.visible = link.visible, link.rank = link.rank, link.overlap = link.overlap, scale = scale, big.gap = big.gap, small.gap = small.gap, ...)
 		}
-	} else {
-		stop("`x` can only be a matrix or a data frame.")
-	}
-	
+	}	
 }
 
 # returns a list, each list containing settings for each new track
@@ -177,7 +181,7 @@ parsePreAllocateTracksValue = function(preAllocateTracks) {
 			return(list(lt2))
 		}
 	} else {
-		stop("Wrong `preAllocateTracks` value.")
+		stop_wrap("Wrong `preAllocateTracks` value.")
 	}
 }
 
@@ -202,7 +206,7 @@ parsePreAllocateTracksValue = function(preAllocateTracks) {
 				mat[ value[i, 1], value[i, 2] ] = value[i, 3]
 			}
 		} else {
-			stop("If ", var_name, " is set as a data frame, it should have three columns.")
+			stop_wrap("If ", var_name, " is set as a data frame, it should have three columns.")
 		}
 	} else if(is.atomic(value) && length(value) == 1) {
 		mat[,] = value
@@ -219,7 +223,7 @@ parsePreAllocateTracksValue = function(preAllocateTracks) {
 				rownames(mat) = rn
 				colnames(mat) = cn
 			} else {
-				stop("If ", var_name, " is a matrix, it should have both rownames and colnames.")
+				stop_wrap("If ", var_name, " is a matrix, it should have both rownames and colnames.")
 			}
 		}
 	}
@@ -330,6 +334,7 @@ mat2df = function(mat) {
 # -link.visible whether plot the link. The value is logical, if it is set to ``FALSE``, the corresponding link will not 
 #            plotted, but the space is still ocuppied. The format of this argument is same as ``link.lwd``
 # -link.rank order to add links to the circle, a large value means to add it later.
+# -link.overlap if it is a directional Chord Diagram, whether the links that come or end in a same sector overlap?
 # -scale scale each sector to same width
 # -big.gap Gap between row sectors and column sectors.
 # -small.gap Small gap between sectors.
@@ -355,10 +360,11 @@ chordDiagramFromMatrix = function(mat, grid.col = NULL, grid.border = NA, transp
 	link.arr.type = "triangle", link.arr.lty = par("lty"), 
 	link.arr.lwd = par("lwd"), link.arr.col = par("col"), 
 	link.largest.ontop = FALSE, link.visible = TRUE, 
-	link.rank = NULL, scale = FALSE, big.gap = 10, small.gap = 1, ...) {
+	link.rank = NULL, link.overlap = FALSE,
+	scale = FALSE, big.gap = 10, small.gap = 1, ...) {
 	
 	if(!is.matrix(mat)) {
-		stop("`mat` can only be a matrix.")
+		stop_wrap("`mat` can only be a matrix.")
 	}
 
 	if(length(mat) != 2) {
@@ -375,7 +381,7 @@ chordDiagramFromMatrix = function(mat, grid.col = NULL, grid.border = NA, transp
 
 	if(symmetric) {
 		if(nrow(mat) != ncol(mat)) {
-			stop("`mat` should be a square matrix.")
+			stop_wrap("`mat` should be a square matrix.")
 		}
 
 		for(i in 1:10) {
@@ -393,7 +399,7 @@ chordDiagramFromMatrix = function(mat, grid.col = NULL, grid.border = NA, transp
 		}
 
 		if(!setequal(rownames(mat), colnames(mat))) {
-			stop("Since you specified a symmetric matrix, rownames and colnames should be the same.")
+			stop_wrap("Since you specified a symmetric matrix, rownames and colnames should be the same.")
 		}
 
 		mat[upper.tri(mat, diag = !keep.diagonal)] = 0
@@ -404,10 +410,10 @@ chordDiagramFromMatrix = function(mat, grid.col = NULL, grid.border = NA, transp
 
 	if(!is.null(order)) {
 		if(is.null(rownames(mat)) || is.null(colnames(mat))) {
-			stop("Since you specified `order`, your matrix should have rowname and colname.")
+			stop_wrap("Since you specified `order`, your matrix should have rowname and colname.")
 		}
 		if(!setequal(order, union(rownames(mat), colnames(mat)))) {
-			stop("Elements in `order` should be same as in `union(rownames(mat), colnames(mat))`.")
+			stop_wrap("Elements in `order` should be same as in `union(rownames(mat), colnames(mat))`.")
 		}
 	}
 	
@@ -439,7 +445,18 @@ chordDiagramFromMatrix = function(mat, grid.col = NULL, grid.border = NA, transp
 	keep_index = names(xlim)[xlim / sum(xlim) >= reduce]
 	ri = which(rownames(mat) %in% keep_index)
 	ci = which(colnames(mat) %in% keep_index)
-	
+
+	ri_zero_sum = ri[ rowSums(mat[ri, ci, drop = FALSE]) == 0]
+	ci_zero_sum = ci[ colSums(mat[ri, ci, drop = FALSE]) == 0]
+
+	while(length(ri_zero_sum) || length(ci_zero_sum)) {
+		ri = setdiff(ri, ri_zero_sum)
+		ci = setdiff(ci, ci_zero_sum)
+
+		ri_zero_sum = ri[ rowSums(mat[ri, ci, drop = FALSE]) == 0]
+		ci_zero_sum = ci[ colSums(mat[ri, ci, drop = FALSE]) == 0]
+	}
+
 	# if the matrix is reduced
 	if(sum(length(ri) + length(ci)) < sum(ncol(mat) + nrow(mat))) {
 		
@@ -590,6 +607,7 @@ chordDiagramFromMatrix = function(mat, grid.col = NULL, grid.border = NA, transp
 		link.largest.ontop = link.largest.ontop,
 		link.visible = link.visible,
 		link.rank = link.rank,
+		link.overlap = link.overlap,
 		scale = scale,
 		big.gap = big.gap,
 		small.gap = small.gap,
@@ -653,6 +671,7 @@ chordDiagramFromMatrix = function(mat, grid.col = NULL, grid.border = NA, transp
 # -link.visible whether plot the link. The value is logical, if it is set to ``FALSE``, the corresponding link will not 
 #            plotted, but the space is still ocuppied. The format of this argument is same as ``link.lwd``
 # -link.rank order to add links to the circle, a large value means to add it later.
+# -link.overlap if it is a directional Chord Diagram, whether the links that come or end in a same sector overlap?
 # -scale scale each sector to same width
 # -big.gap Gaps between the sectors in the first column of ``df`` and sectors in the second column in ``df``.
 # -small.gap Small gap between sectors.
@@ -678,8 +697,8 @@ chordDiagramFromDataFrame = function(df, grid.col = NULL, grid.border = NA, tran
 	link.arr.lwd = par("lwd"), link.arr.col = par("col"), 
 	link.largest.ontop = FALSE, link.visible = TRUE, 
 	link.rank = seq_len(nrow(df)), 
-	scale = FALSE, 
-	big.gap = 10, small.gap = 1,
+	link.overlap = FALSE,
+	scale = FALSE, big.gap = 10, small.gap = 1,
 	...) {
 
 	if(nrow(df) != 2) {
@@ -690,11 +709,11 @@ chordDiagramFromDataFrame = function(df, grid.col = NULL, grid.border = NA, tran
 
 	# check the format of the data frame
 	if(!inherits(df, "data.frame")) {
-		stop("`df` must be a data frame.")
+		stop_wrap("`df` must be a data frame.")
 	}
 	df = as.data.frame(df)
 	if(ncol(df) < 2) {
-		stop("`df` should have at least have two columns.")
+		stop_wrap("`df` should have at least have two columns.")
 	}
 	if(ncol(df) == 2) {
 		df[, 3] = rep(1, nrow(df))
@@ -724,16 +743,16 @@ chordDiagramFromDataFrame = function(df, grid.col = NULL, grid.border = NA, tran
 	if(!is.null(order)) {
 		order = intersect(order, cate)
 		if(length(order) != length(cate)) {
-			stop("`order` should contain names of all sectors.")
+			stop_wrap("`order` should contain names of all sectors.")
 		}
 		if(is.numeric(order)) {
 			if(!setequal(order, seq_along(cate))) {
-				stop("`order` needs to be integers ranging from 1 to", length(cate))
+				stop_wrap("`order` needs to be integers ranging from 1 to", length(cate))
 			}
 			cate = cate[order]
 		} else {
 			if(!setequal(order, cate)) {
-				stop("`order` should only be picked from sectors.")	
+				stop_wrap("`order` should only be picked from sectors.")	
 			}
 			cate = order
 		}
@@ -818,41 +837,48 @@ chordDiagramFromDataFrame = function(df, grid.col = NULL, grid.border = NA, tran
 
 
 	#### reduce the data frame
-	xsum = structure(rep(0, length(cate)), names = cate)
-	for(i in seq_len(nr)) {
-		if(df$rn[i] == df$cn[i]) {
-			xsum[df$rn[i]] = xsum[df$rn[i]] + abs(df$value1[i])
-			if(self.link == 2) {
-				xsum[df$rn[i]] = xsum[df$rn[i]] + abs(df$value2[i])  # <<- self-link!!!!!
+	onr = nrow(df)
+	while(1) {
+		xsum = structure(rep(0, length(cate)), names = cate)
+		for(i in seq_len(nr)) {
+			if(df$rn[i] == df$cn[i]) {
+				xsum[df$rn[i]] = xsum[df$rn[i]] + abs(df$value1[i])
+				if(self.link == 2) {
+					xsum[df$rn[i]] = xsum[df$rn[i]] + abs(df$value2[i])  # <<- self-link!!!!!
+				}
+			} else {
+				xsum[df$rn[i]] = xsum[df$rn[i]] + abs(df$value1[i])
+				xsum[df$cn[i]] = xsum[df$cn[i]] + abs(df$value2[i])
 			}
-		} else {
-			xsum[df$rn[i]] = xsum[df$rn[i]] + abs(df$value1[i])
-			xsum[df$cn[i]] = xsum[df$cn[i]] + abs(df$value2[i])
 		}
+
+		keep = names(xsum)[xsum / sum(xsum) >= reduce]
+		l = df$rn %in% keep & df$cn %in% keep
+
+		cate = intersect(cate, keep)
+		df = df[l, , drop = FALSE]
+		grid.col = grid.col[intersect(names(grid.col), keep)]
+		col = col[l]
+		link.border = link.border[l]
+		link.lwd = link.lwd[l]
+		link.lty = link.lty[l] 
+		link.arr.length = link.arr.length[l]
+		link.arr.width = link.arr.width[l]
+		link.arr.type = link.arr.type[l]
+		link.arr.lwd = link.arr.lwd[l]
+		link.arr.lty = link.arr.lty[l]
+		link.arr.col = link.arr.col[l]
+		link.visible = link.visible[l]
+		link.rank = link.rank[l]
+		directional = directional[l]
+		direction.type = direction.type[l]
+
+		nr = nrow(df)
+		reduce = 1e-10
+		if(nr == onr) break
+		onr = nr
 	}
 
-	keep = names(xsum)[xsum / sum(xsum) >= reduce]
-	l = df$rn %in% keep & df$cn %in% keep
-
-	cate = intersect(cate, keep)
-	df = df[l, , drop = FALSE]
-	grid.col = grid.col[intersect(names(grid.col), keep)]
-	col = col[l]
-	link.border = link.border[l]
-	link.lwd = link.lwd[l]
-	link.lty = link.lty[l] 
-	link.arr.length = link.arr.length[l]
-	link.arr.width = link.arr.width[l]
-	link.arr.type = link.arr.type[l]
-	link.arr.lwd = link.arr.lwd[l]
-	link.arr.lty = link.arr.lty[l]
-	link.arr.col = link.arr.col[l]
-	link.visible = link.visible[l]
-	link.rank = link.rank[l]
-	directional = directional[l]
-	direction.type = direction.type[l]
-
-	nr = nrow(df)
 	# re-calcualte xsum
 	xsum = structure(rep(0, length(cate)), names = cate)
 	for(i in seq_len(nr)) {
@@ -974,6 +1000,12 @@ chordDiagramFromDataFrame = function(df, grid.col = NULL, grid.border = NA, tran
 		}
 	}
 
+	if(link.overlap) {
+		x1_sum = tapply(df$x1, df$rn, max)[names(xsum)]
+		x2_sum = tapply(df$x2, df$cn, max)[names(xsum)] - x1_sum
+		df$x2 = df$x2 - x1_sum[df$cn]
+		xsum = pmax(x1_sum, x2_sum)
+	}
 
 	o.cell.padding = circos.par("cell.padding")
 	circos.par(cell.padding = c(0, 0, 0, 0))
